/*
 * random functions
 *
 * author: hackbard@hackdaworld.dyndns.org
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>

#ifdef USE_LIBC_RAND
#include <time.h>
#endif

#include "random.h"

static int rand_fd;
static u32 *c_ptr,*b_ptr;

int rand_init(char *rf)
{
 if(rf==NULL)
 {
#ifndef USE_LIBC_RAND
  if((rand_fd=open("/dev/urandom",O_RDONLY))<0)
  {
   puts("cannot open /dev/urandom");
   return -1;
  }
#endif
 } else
 {
  if((rand_fd=open(rf,O_RDONLY))<0)
  {
   printf("cannot open %s\n",rf);
   return -1;
  }
 }
 if((b_ptr=(u32 *)(malloc(BUFSIZE*sizeof(u32))))==NULL)
 {
  puts("failed allocating random buffer");
  return -1;
 }
 c_ptr=b_ptr+BUFSIZE;
 
 return 1;
}

int rand_close(void)
{
#ifndef USE_LIBC_RAND
 close(rand_fd);
#endif
 
 return 1;
}

u32 get_rand(u32 max)
{
 if(c_ptr>=b_ptr+BUFSIZE)
 {
#ifdef MORE_PRINTF
  printf("getting another %d bytes of random data ...\n",BUFSIZE);
#endif
#ifdef USE_LIBC_RAND
  c_ptr=b_ptr;
  srand((int)time(NULL));
  while(c_ptr<b_ptr+BUFSIZE) *(c_ptr++)=(u32)rand();
  c_ptr=b_ptr;
#else
  if(read(rand_fd,b_ptr,BUFSIZE*sizeof(u32))<BUFSIZE*sizeof(u32))
  {
   /* -> assume random file, end reached */
   puts("warning, will read random file from beginning ...");
   lseek(rand_fd,0,SEEK_SET);
   read(rand_fd,b_ptr,BUFSIZE*sizeof(u32));
  }
  c_ptr=b_ptr;
#endif
#ifdef MORE_PRINTF
  printf("got it!\n");
#endif
 }

#ifdef USE_LIBC_RAND
 return((u32)(*(c_ptr++)*(max*1.0/((long long unsigned int)RAND_MAX+1))));
#else
 return((u32)(*(c_ptr++)*(max*1.0/((long long unsigned int)URAND_MAX+1))));
#endif
}

u32 get_rand_lgp(u32 max,double a,double b)
{
 return((u32)(1.0*max*(-1.0*b+sqrt(b*b+2*a*((b+a/2)*get_rand(URAND_MAX)/((long long unsigned int)URAND_MAX+1))))/a));
}

u32 get_rand_reject(u32 max_x,u32 max_y,u32 *graph)
{
 u32 x,y;
 unsigned char ok;

 ok=0;
 while(!ok)
 {
  x=get_rand(max_x);
  y=get_rand(max_y);
  if(y<=graph[x]) ok=1;
 }

 return x;
}
